@@ -6,10 +6,15 @@
 # $Id: wfs.py 503 2006-02-01 17:09:12Z dokai $
 # =============================================================================
 
+from io import BytesIO
+import logging
+from urllib.parse import urlencode
+
 # owslib imports:
 from owslib import util
 from owslib.fgdc import Metadata
 from owslib.iso import MD_Metadata
+from owslib.iso3 import MD_Metadata as MD_Metadata3  # ISO 19115 Part 3 XML
 from owslib.ows import Constraint, ServiceIdentification, ServiceProvider, OperationsMetadata
 from owslib.etree import etree
 from owslib.util import nspath, testXMLValue, openURL, Authentication
@@ -17,16 +22,11 @@ from owslib.crs import Crs
 from owslib.feature import WebFeatureService_
 from owslib.feature.common import (
     WFSCapabilitiesReader,
-    AbstractContentMetadata,
+    AbstractContentMetadata
 )
 from owslib.namespaces import Namespaces
 
-# other imports
-from io import BytesIO
-from urllib.parse import urlencode
-
-import logging
-from owslib.util import log
+LOGGER = logging.getLogger(__name__)
 
 n = Namespaces()
 WFS_NAMESPACE = n.get_namespace("wfs20")
@@ -45,48 +45,6 @@ class WebFeatureService_2_0_0(WebFeatureService_):
 
     Implements IWebFeatureService.
     """
-
-    def __new__(
-        self,
-        url,
-        version,
-        xml,
-        parse_remote_metadata=False,
-        timeout=30,
-        headers=None,
-        username=None,
-        password=None,
-        auth=None,
-    ):
-        """ overridden __new__ method
-
-        @type url: string
-        @param url: url of WFS capabilities document
-        @type xml: string
-        @param xml: elementtree object
-        @type parse_remote_metadata: boolean
-        @param parse_remote_metadata: whether to fully process MetadataURL elements
-        @param headers: HTTP headers to send with requests
-        @param timeout: time (in seconds) after which requests should timeout
-        @param username: service authentication username
-        @param password: service authentication password
-        @param auth: instance of owslib.util.Authentication
-        @return: initialized WebFeatureService_2_0_0 object
-        """
-        obj = object.__new__(self)
-        obj.__init__(
-            url,
-            version,
-            xml,
-            parse_remote_metadata,
-            timeout,
-            headers=headers,
-            username=username,
-            password=password,
-            auth=auth,
-        )
-        return obj
-
     def __getitem__(self, name):
         """ check contents dictionary to allow dict like access to service layers"""
         if name in list(self.__getattribute__("contents").keys()):
@@ -115,7 +73,7 @@ class WebFeatureService_2_0_0(WebFeatureService_):
         else:
             auth = Authentication()
         super(WebFeatureService_2_0_0, self).__init__(auth)
-        log.debug("building WFS %s" % url)
+        LOGGER.debug("building WFS %s" % url)
         self.url = url
         self.version = version
         self.timeout = timeout
@@ -226,6 +184,7 @@ class WebFeatureService_2_0_0(WebFeatureService_):
         featureversion=None,
         propertyname=None,
         maxfeatures=None,
+        srsname=None,
         storedQueryID=None,
         storedQueryParams=None,
         method="Get",
@@ -254,6 +213,8 @@ class WebFeatureService_2_0_0(WebFeatureService_):
             For Post request, leave blank (None) to get all properties.
         maxfeatures : int
             Maximum number of features to be returned.
+        srsname: string
+            EPSG code to request the data in
         storedQueryID : string
             A name identifying a prepared set available in WFS-service
         storedQueryParams : dict
@@ -298,6 +259,7 @@ class WebFeatureService_2_0_0(WebFeatureService_):
                 featureversion,
                 propertyname,
                 maxfeatures,
+                srsname,
                 storedQueryID,
                 storedQueryParams,
                 outputFormat,
@@ -305,7 +267,7 @@ class WebFeatureService_2_0_0(WebFeatureService_):
                 startindex,
                 sortby,
             )
-            log.debug("GetFeature WFS GET url %s" % url)
+            LOGGER.debug("GetFeature WFS GET url %s" % url)
         else:
             url, data = self.getPOSTGetFeatureRequest(
                 typename,
@@ -605,11 +567,20 @@ class ContentMetadata(AbstractContentMetadata):
 
                     mdelem = doc.find(
                         ".//" + util.nspath_eval("gmd:MD_Metadata", n.get_namespaces(["gmd"]))
-                    ) or doc.find(
-                        ".//" + util.nspath_eval("gmi:MI_Metadata", n.get_namespaces(["gmi"]))
                     )
+                    if mdelem is None:
+                        mdelem = doc.find(
+                            ".//" + util.nspath_eval("gmi:MI_Metadata", n.get_namespaces(["gmi"]))
+                        )
                     if mdelem is not None:
                         metadataUrl["metadata"] = MD_Metadata(mdelem)
                         continue
+                    else:  # ISO 19115 Part 3 XML
+                        mdelem = MD_Metadata3.find_start(doc)
+                        if mdelem is not None:
+                            metadataUrl["metadata"] = MD_Metadata3(mdelem)
+                        else:
+                            metadataUrl["metadata"] = None
+                        continue
                 except Exception:
                     metadataUrl["metadata"] = None
